/*
 * This file is part of the libsigrok project.
 *
 * Copyright (C) {year} {author} <{email}>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include "protocol.h"

SR_PRIV struct sr_dev_driver {lib}_driver_info;

static int init(struct sr_dev_driver *di, struct sr_context *sr_ctx)
{{
	return std_init(sr_ctx, di, LOG_PREFIX);
}}

static GSList *scan(struct sr_dev_driver *di, GSList *options)
{{
	struct drv_context *drvc;
	GSList *devices;

	(void)options;

	devices = NULL;
	drvc = di->context;
	drvc->instances = NULL;

	/* TODO: scan for devices, either based on a SR_CONF_CONN option
	 * or on a USB scan. */

	return devices;
}}

static GSList *dev_list(const struct sr_dev_driver *di)
{{
	return ((struct drv_context *)(di->context))->instances;
}}

static int dev_clear(const struct sr_dev_driver *di)
{{
	return std_dev_clear(di, NULL);
}}

static int dev_open(struct sr_dev_inst *sdi)
{{
	(void)sdi;

	/* TODO: get handle from sdi->conn and open it. */

	sdi->status = SR_ST_ACTIVE;

	return SR_OK;
}}

static int dev_close(struct sr_dev_inst *sdi)
{{
	(void)sdi;

	/* TODO: get handle from sdi->conn and close it. */

	sdi->status = SR_ST_INACTIVE;

	return SR_OK;
}}

static int cleanup(const struct sr_dev_driver *di)
{{
	dev_clear(di);

	/* TODO: free other driver resources, if any. */

	return SR_OK;
}}

static int config_get(uint32_t key, GVariant **data,
	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
{{
	int ret;

	(void)sdi;
	(void)data;
	(void)cg;

	ret = SR_OK;
	switch (key) {{
	/* TODO */
	default:
		return SR_ERR_NA;
	}}

	return ret;
}}

static int config_set(uint32_t key, GVariant *data,
	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
{{
	int ret;

	(void)data;
	(void)cg;

	if (sdi->status != SR_ST_ACTIVE)
		return SR_ERR_DEV_CLOSED;

	ret = SR_OK;
	switch (key) {{
	/* TODO */
	default:
		ret = SR_ERR_NA;
	}}

	return ret;
}}

static int config_list(uint32_t key, GVariant **data,
	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
{{
	int ret;

	(void)sdi;
	(void)data;
	(void)cg;

	ret = SR_OK;
	switch (key) {{
	/* TODO */
	default:
		return SR_ERR_NA;
	}}

	return ret;
}}

static int dev_acquisition_start(const struct sr_dev_inst *sdi, void *cb_data)
{{
	(void)sdi;
	(void)cb_data;

	if (sdi->status != SR_ST_ACTIVE)
		return SR_ERR_DEV_CLOSED;

	/* TODO: configure hardware, reset acquisition state, set up
	 * callbacks and send header packet. */

	return SR_OK;
}}

static int dev_acquisition_stop(struct sr_dev_inst *sdi, void *cb_data)
{{
	(void)cb_data;

	if (sdi->status != SR_ST_ACTIVE)
		return SR_ERR_DEV_CLOSED;

	/* TODO: stop acquisition. */

	return SR_OK;
}}

SR_PRIV struct sr_dev_driver {lib}_driver_info = {{
	.name = "{short}",
	.longname = "{name}",
	.api_version = 1,
	.init = init,
	.cleanup = cleanup,
	.scan = scan,
	.dev_list = dev_list,
	.dev_clear = dev_clear,
	.config_get = config_get,
	.config_set = config_set,
	.config_list = config_list,
	.dev_open = dev_open,
	.dev_close = dev_close,
	.dev_acquisition_start = dev_acquisition_start,
	.dev_acquisition_stop = dev_acquisition_stop,
	.context = NULL,
}};

SR_REGISTER_DEV_DRIVER({lib}_driver_info);
